* util.prg - Andrew Bedno - 1993
* Miscellaneous xBase utility routines.

* Initialize once before using any library routines.
PROCEDURE lib_init
  PUBLIC NL
  STORE CHR(13) + CHR(10) TO NL
  PUBLIC debug_level
  PUBLIC debug_msg
  PUBLIC m_escape
  STORE 0 TO debug_level
  STORE '' TO debug_msg
  STORE .F. TO m_escape
  * Used by print manager.
  PUBLIC m_maxline, m_pgcnt, m_pgfoot
  PUBLIC m_prtline, m_prtcntd
  PUBLIC m_hd1, m_hd2, m_hd3, m_hd4, m_hd5
  PUBLIC m_dest, m_livedest, m_destname, m_destmsg, m_outfile, m_outln
  STORE 66 TO m_maxline
  STORE 8 TO m_pgfoot
  * Customize these system behavior settings if desired.
  ON ESCAPE DO esc_trap 
  ON ERROR DO err_trap 
  SET SCOREBOARD OFF
  SET STATUS OFF
  SET EXCLUSIVE OFF
  SET DELETED OFF
RETURN


* Trap errors.
PROCEDURE err_trap
  @ 0, 0 SAY 'Error #' + ALLTRIM(STR(ERROR())) + ' has occurred.'
  BEEP
  STORE .T. TO m_escape
RETURN


* Trap escape key.
PROCEDURE esc_trap
  STORE .T. TO m_escape
RETURN


* Optionally display debug message.
PROCEDURE dbgmsg
  IF debug_level > 0
    IF .NOT. CONFIRM( debug_msg )
      RETURN TO MASTER
    ENDIF
  ENDIF
RETURN

* PRINTING WITH PRINTMGR:
* Step 0:
*   System initialization sets up page information:
*     STORE 6(lpi) * 11(inch) TO m_maxline  
*     STORE 6(lpi) + 2 TO m_pgfoot
* Step 1:
*   Store file name (w/o path or ext) to m_destname before choose/setup.
* Step 2:
*   Call prn_choose or prn_setup:
*     prn_choose:
*       Displays output destination dialog box.
*       Sets m_dest to S/F/P or sets m_escape.
*     prn_setup:
*       Expects m_dest already set to S/F/P.
*     Stores 0 to m_prtline.
*     Stores 1 to m_pgcnt.
* Step 3:
*   Store new values to m_maxline and m_pgfoot if desired.
*   Call prn_open.
*   A printing message will be displayed.
*   Call a display status routine if desired.
* Step 4:
*   IF m_prtline = 0 print program specific page header.
*     (note that m_hd1..5 are available for general use)
*   Store text to m_outln, call prn_write.
*   Adds 1 to m_prtline.
*   Call an update status display routine if desired.
* Step 5:
*   Handle page length:
*     IF m_prtline >= m_maxline - m_pgfoot DO formfeed.
*     Stores 0 to m_prtline.
*     Adds 1 to m_pgcnt.
* Step 6:
*   If m_prtline > 0 DO formfeed
*   Call a close status display routine if desired.
*   Call prn_close.

* Perform various output device control functions.
PROCEDURE printmgr
  PARAMETERS m_prtfunc
  PRIVATE m_scrnnm, m_prnnum, btn_tmp
  * 0=prompt, 1=open, 2=close, 3=write, 4=setup/no prompt
  IF ( m_prtfunc = prn_choose ) .OR. ( m_prtfunc = prn_setup )
    IF m_prtfunc = prn_choose
      STORE ' ' TO m_dest
    ENDIF
    STORE ' ' TO m_livedest
    STORE 0 TO m_outfile
    IF m_prtfunc = prn_choose
      STORE '' TO btn_tmp
      DO WHILE (btn_tmp <> 'P') .AND. (btn_tmp <> 'F') ;
               .AND. (btn_tmp <> 'S')
        WAIT 'Output destination? <P>rinter/<F>ile/<S>creen: ' TO btn_tmp
        STORE UPPER(btn_tmp) TO btn_tmp
      ENDDO
      STORE btn_tmp TO m_dest      
    ENDIF
    STORE 1 TO m_pgcnt
    IF .NOT. m_escape
      IF m_dest = 'P'
        STORE 'PRINTER' TO m_destmsg
        STORE cfg_prn TO m_destname
      ENDIF
      IF m_dest = 'S'
        STORE 'SCREEN' TO m_destmsg
        STORE 'SCREEN.TXT' TO m_destname
      ENDIF
      IF m_dest = 'F'
        STORE 'FILE' TO m_destmsg
        STORE m_destname + '.TXT' TO m_destname
      ENDIF
    ENDIF
  ENDIF
  IF m_prtfunc = prn_open
    STORE 0 TO m_prnnum
* Check if printer is ready before opening.
*   DO CASE
*     CASE '.'+UPPER(SUBSTR(cfg_prn,1,4))+'.' = '.LPT1.'
*       STORE 1 TO m_prnnum
*     CASE '.'+UPPER(SUBSTR(cfg_prn,1,4))+'.' = '.LPT2.'
*       STORE 2 TO m_prnnum
*     CASE '.'+UPPER(SUBSTR(cfg_prn,1,4))+'.' = '.COM1.'
*       STORE 3 TO m_prnnum
*   ENDCASE
*   DO WHILE m_prnnum > 0
*     IF ( BITAND(PRNTSTAT(m_prnnum),41) > 0 ) .OR. ;
*        ( BITAND(PRNTSTAT(m_prnnum),144) <> 144 )
*       WAIT 'Printer is not ready.  Press <Y> to retry: ' TO btn_tmp
*       STORE UPPER(btn_tmp) TO btn_tmp
*       IF btn_tmp <> 'Y'
*         STORE -1 TO m_prnnum
*         STORE ' ' TO m_dest
*         STORE ' ' TO m_livedest
*         STORE .T. TO m_escape
*       ENDIF
*     ELSE
*       STORE 0 TO m_prnnum
*     ENDIF
*   ENDDO
    IF m_prnnum >= 0
      STORE 0 TO m_prtline
      STORE m_dest TO m_livedest
      IF m_livedest = 'P'
        STORE 'Printing . . .' TO m_msg
      ENDIF
      IF m_livedest = 'S'
        STORE 'Preparing screen output . . .' TO m_msg
      ENDIF
      IF m_livedest = 'F'
        STORE 'Writing to <' + cfg_lpath + m_destname + '> . . .' TO m_msg
      ENDIF
      DO prompt WITH m_msg
      SET EXCLUSIVE ON
      * First open always seems to return an error.
      IF m_livedest = 'P'
        STORE FCREATE(cfg_prn) TO m_outfile
        IF m_outfile < 1
          STORE FCREATE(cfg_prn) TO m_outfile
        ENDIF
      ELSE
        STORE FCREATE(cfg_lpath + m_destname) TO m_outfile
        IF m_outfile < 1
          STORE FCREATE(cfg_lpath + m_destname) TO m_outfile
        ENDIF
      ENDIF
      SET EXCLUSIVE OFF
      IF m_outfile < 1
        STORE ' ' TO m_dest
        STORE ' ' TO m_livedest
        BEEP
        STORE .T. TO m_escape
        WAIT 'Cannot open output file!' TO btn_tmp
      ENDIF
    ENDIF
  ENDIF
  IF m_prtfunc = prn_close
    STORE ' ' TO m_dest
    IF m_livedest <> ' '
      IF m_outfile > 0
        FCLOSE( m_outfile )
      ENDIF
      IF m_livedest = 'S'
        STORE cfg_lpath + m_destname TO m_scrnnm
        WRUN &cfg_note &m_scrnnm
      ENDIF
      STORE 0 TO m_outfile
      STORE ' ' TO m_livedest
    ENDIF
  ENDIF
  IF m_prtfunc = prn_write
    IF m_outfile > 0
      IF m_outln = FF
        STORE FWRITE( m_outfile, @m_outln, 1 ) TO m_fputstat
      ELSE
        STORE FPUTS( m_outln, m_outfile ) TO m_fputstat
      ENDIF
      STORE m_prtline + 1 TO m_prtline
      IF m_fputstat < LEN( m_outln )
        STORE ' ' TO m_dest
        STORE ' ' TO m_livedest
        FCLOSE( m_outfile )
        STORE 0 TO m_outfile
        BEEP
        STORE .T. TO m_escape
        WAIT 'Trouble writing output!' TO btn_tmp
      ENDIF
    ENDIF
  ENDIF
  RELEASE m_scrnnm, m_prnnum
RETURN


* Eject lines until positioned on first line of next page.
PROCEDURE formfeed
  DO WHILE m_prtline < m_maxline
    STORE '' TO m_outln
    DO printmgr WITH prn_write
  ENDDO
  STORE 0 TO m_prtline
  STORE m_pgcnt + 1 TO m_pgcnt
RETURN


* Format phone number as (aaa)xxx-yyyy
PROCEDURE fmtphone
  * Accepts and returns public fmt_phone variable.
  PRIVATE fmt_phsave, fmt_phch, fmt_phtmp, fmt_phac
  STORE '' TO fmt_phtmp
  STORE fmt_phone TO fmt_phsave
  * Reduce to numbers only.
  DO WHILE LEN( fmt_phone ) > 0
    STORE SUBSTR( fmt_phone, 1, 1 ) TO fmt_phch
    IF ( fmt_phch >= '0' ) .AND. ( fmt_phch <= '9' )
      STORE fmt_phtmp + fmt_phch TO fmt_phtmp
    ENDIF
    STORE SUBSTR( fmt_phone, 2, LEN( fmt_phone ) - 1 ) TO fmt_phone
  ENDDO
  IF ( LEN(fmt_phtmp) <> 7 ) .AND. ( LEN(fmt_phtmp) <> 10 )
    STORE fmt_phsave TO fmt_phone
  ELSE
    IF LEN(fmt_phtmp) < 10
      STORE cfg_ac TO fmt_phac
    ELSE
      STORE SUBSTR( fmt_phtmp, 1, 3 ) TO fmt_phac
      STORE SUBSTR( fmt_phtmp, 4, 7 ) TO fmt_phtmp
    ENDIF
    IF LEN(ALLTRIM(cfg_ac)) < 1
      STORE '' TO fmt_phone
    ELSE
      STORE '(' + fmt_phac + ')' TO fmt_phone
    ENDIF
    STORE fmt_phone + SUBSTR( fmt_phtmp, 1, 3 ) + '-' + ;
          SUBSTR( fmt_phtmp, 4, 4 ) TO fmt_phone
  ENDIF
  RELEASE fmt_phsave, fmt_phch, fmt_phtmp, fmt_phac
RETURN


* Example routine to pack current database and recreate index.
PROCEDURE db_fix
  PARAMETERS m_iname
  PRIVATE mn_oname
  SET EXCLUSIVE ON
  SET TALK ON
  @ 0, 0 SAY 'Packing ' + m_iname + ' . . .'
  PACK
  @ 0, 0 SAY 'Indexing ' + m_iname + ' . . .'
* Customize some of these lines, then uncomment:
* STORE m_iname + '.ndx' TO m_oname
* IF FILESIZE(m_oname) > 0
*   ERASE &m_oname
* ENDIF
* INDEX ON ... TO &m_oname
  SET TALK OFF
  SET EXCLUSIVE OFF
  RELEASE m_oname
RETURN


***** THIS IS THE LAST LINE *****
